import { resolve } from "path";
let cachedProcess;
export default async function hidAsyncProcess() {
    if (cachedProcess) {
        return cachedProcess;
    }
    const fork = await import("child_process").then((m) => m.fork);
    const proc = fork(resolve(__dirname, "..", "offload", "hid-process"));
    // tslint:disable-next-line:ban-types
    const funcmods = new Set();
    const mp = {
        process: proc,
        on(event, listener) {
            if (event === "message" && !funcmods.has(event)) {
                const f = (msg) => {
                    if (msg.d && !(msg.d instanceof Uint8Array) && msg.d.type === "Buffer") {
                        msg.d = new Uint8Array(msg.d.data);
                    }
                };
                funcmods.add(event);
                this.process.on(event, f);
            }
            this.process.on(event, listener);
            return this;
        },
        off(event, listener) {
            this.process.off(event, listener);
            return this;
        },
        postMessage(data) {
            if (data && data.d) {
                if (data.d instanceof Uint8Array) {
                    data.d = Array.from(data.d);
                }
                else if (Array.isArray(data.d) && data.d.length) {
                    data.d = data.d.map((x) => x instanceof Uint8Array ? Array.from(x) : x);
                }
            }
            this.process.send(data);
        },
    };
    return cachedProcess = new Promise((res, rej) => {
        const f = (m) => {
            proc.off("message", f);
            if (m && m.t === "ready") {
                res(mp);
            }
            else {
                rej(new Error("Unexpected response: " + JSON.stringify(m)));
            }
        };
        proc.on("message", f);
        proc.once("error", (err) => rej(err));
    });
}
export async function devices() {
    const port = await hidAsyncProcess();
    return new Promise((res) => {
        const f = (msg) => {
            if (msg.t === "d" && msg.d) {
                res(msg.d);
                port.off("message", f);
            }
        };
        port.on("message", f);
        port.postMessage({ t: "d" });
    });
}
//# sourceMappingURL=hid-async-process.js.map