import { resolve } from "path";
let hidCache;
export default async function hidAsyncWorker() {
    if (hidCache) {
        return hidCache;
    }
    const { Worker } = await import("worker_threads");
    const worker = new Worker(resolve(__dirname, "..", "offload", "hid-worker.js"));
    // tslint:disable-next-line:ban-types
    const funcmods = new Set();
    const mp = {
        worker,
        on(event, listener) {
            if (event === "message" && funcmods.has(event)) {
                const f = (msg) => {
                    if (msg.d && !(msg.d instanceof Uint8Array) && msg.d.type === "Buffer") {
                        msg.d = new Uint8Array(msg.d.data);
                    }
                    listener(msg);
                };
                this.worker.on(event, f);
            }
            this.worker.on(event, listener);
            return this;
        },
        off(event, listener) {
            this.worker.off(event, listener);
            return this;
        },
        postMessage(data, transferList) {
            this.worker.postMessage(data, transferList);
        },
    };
    return hidCache = new Promise((res, rej) => {
        const f = (m) => {
            worker.off("message", f);
            if (m && m.t === "ready") {
                res(mp);
            }
            else {
                rej(new Error("Unexpected response: " + JSON.stringify(m)));
            }
        };
        worker.on("message", f);
    });
}
export async function devices() {
    const port = await hidAsyncWorker();
    return new Promise((res) => {
        const f = (msg) => {
            if (msg.t === "d" && msg.d) {
                res(msg.d);
                port.off("message", f);
            }
        };
        port.on("message", f);
        port.postMessage({ t: "d" });
    });
}
//# sourceMappingURL=hid-async-worker.js.map