import { EventEmitter } from "../event-emitter";
import { IPostMessage } from "./hid-messageport";
import { Device } from "node-hid";
export interface IHidAsyncEventEmitter {
    on(event: "data", listener: (data: Uint8Array) => any): this;
}
export declare type HidAsyncType = "auto" | "process" | "worker" | "emulated";
declare const isOpen: unique symbol;
declare const asyncCounter: unique symbol;
declare const connectedDeviceCount: unique symbol;
/**
 * An async controller for a connected HID device.
 */
export declare class HidAsync extends EventEmitter implements IHidAsyncEventEmitter {
    protected readonly devicePath: string;
    private static [asyncCounter];
    private static readonly [connectedDeviceCount];
    protected readonly awaitingAsync: Map<number, [(x: any) => void, (e: any) => void]>;
    protected messagePort: IPostMessage;
    protected [isOpen]: boolean;
    constructor(devicePath: string);
    /**
     * Read data from the device.
     */
    read(): Promise<Uint8Array>;
    open(): boolean;
    /**
     * Closes the reference to this device.
     */
    close(): false | undefined | Promise<true>;
    /**
     * Write data to the device.
     * @param {Uint8Array} data data to asynchronously write to the device.
     */
    write(data: Uint8Array): Promise<number>;
    /**
     * Write multiple data blocks to the device.
     * @param {Uint8Array[]} datas data blocks to asynchronously write to the device.
     */
    writeMulti(datas: Uint8Array[]): Promise<number>;
    /**
     * Write data to the device.
     * @param {Uint8Array} data data to asynchronously write to the device.
     */
    sendFeatureReport(data: Uint8Array): Promise<number>;
    protected postMessageAsync<T extends {}, R>(data: T, transferList?: ArrayBuffer[]): Promise<R>;
    protected receiveMessage(data: any): void;
}
export declare function setHidAsyncType(type: HidAsyncType): void;
/**
 * Get a list of all devices.
 */
export declare function devices(): Promise<Device[]>;
export {};
