import { HidContainer } from "./hid-container";
// tslint:disable-next-line:ban-types
const funcMap = new WeakMap();
const containers = new Map();
const parentPort = {
    on(event, listener) {
        let f = funcMap.get(listener);
        if (!f) {
            f = (msg) => {
                if (msg.d && !(msg.d instanceof Uint8Array) && msg.d.type === "Buffer") {
                    msg.d = msg.d.data;
                }
                listener(msg);
            };
            funcMap.set(listener, f);
        }
        process.on(event, f);
        return this;
    },
    off(event, listener) {
        const f = funcMap.get(listener) || listener;
        process.off(event, f);
        return this;
    },
    postMessage(data) {
        if (!process.send) {
            throw new Error("process.send is undefined");
        }
        process.send(data);
    },
};
parentPort.on("message", (msg) => {
    if (!msg.p && msg.t === "n" && typeof msg.d === "string" && !containers.has(msg.d)) {
        const hid = new HidContainer(msg.d, parentPort);
        containers.set(msg.d, hid);
        hid.init();
    }
    else if (!msg.p && msg.t === "d") {
        import("node-hid").then((hid) => {
            parentPort.postMessage({ t: "d", d: hid.devices() });
        });
    }
});
//# sourceMappingURL=hid-process.js.map