export interface IImageLibraryExtract {
    top: number;
    left: number;
    height: number;
    width: number;
}
export interface IImageLibrary {
    /**
     * Extract a section of the image.
     *
     * If this is a immutable data structure a clone of the original should be returned with the new 2d slice.
     * A mutable structure should return the original but with a view which may be moved with another call to extract.
     *
     * @param {number} [opt.top] the distance from the top edge of the original
     * @param {number} [opt.left] the distance from the left edge of the original
     * @param {number} [opt.height] the height of this subsection
     * @param {number} [opt.width] the width of this subsection
     */
    extract(opt: IImageLibraryExtract): IImageLibrary | PromiseLike<IImageLibrary>;
    /** Remove alpha channel if one exists. */
    flatten(): IImageLibrary | PromiseLike<IImageLibrary>;
    /** Resize an image to a specific size. */
    resize(width: number, height: number): IImageLibrary | PromiseLike<IImageLibrary>;
    /** Get the RGB raw pixel bytes representing the image. */
    toUint8Array(): Uint8Array | PromiseLike<Uint8Array>;
}
export interface IImageRawOptions {
    channels: 1 | 2 | 3 | 4;
    height: number;
    width: number;
}
export interface IImageLibraryCreator {
    loadFile(filepath: string): IImageLibrary | PromiseLike<IImageLibrary>;
    loadFileData(filedata: Uint8Array): IImageLibrary | PromiseLike<IImageLibrary>;
    createRaw(data: Uint8Array, rawOptions: IImageRawOptions): IImageLibrary | PromiseLike<IImageLibrary>;
}
declare const _default: IImageLibraryCreator;
export default _default;
