async function defLib() {
    const sharpi = await import("sharp");
    return typeof sharpi === "function" ? sharpi : sharpi.default;
}
const lib = {
    _defLib: null,
    _sharpProxy(img) {
        return new Proxy(img, {
            get(target, p, receiver) {
                if (p === "toUint8Array") {
                    return () => target.raw().toBuffer();
                }
                // mutable actions returning `this`.
                if (p === "extract" || p === "flatten" || p === "resize") {
                    return (...args) => {
                        target[p].apply(target, args);
                        return receiver;
                    };
                }
                return target[p];
            },
        });
    },
    async _load(file) {
        if (!this._defLib) {
            this._defLib = defLib();
        }
        const Lib = await this._defLib;
        const x = Lib(file, { density: 300 });
        return this._sharpProxy(x);
    },
    loadFile(filepath) {
        return this._load(filepath);
    },
    loadFileData(filedata) {
        return this._load(Buffer.isBuffer(filedata) ? filedata : Buffer.from(filedata));
    },
    async createRaw(data, rawOptions) {
        if (!this._defLib) {
            this._defLib = defLib();
        }
        const d = Buffer.isBuffer(data) ? data : Buffer.from(data);
        const Lib = await this._defLib;
        const x = Lib(d, { raw: rawOptions });
        return this._sharpProxy(x);
    },
};
export default lib;
//# sourceMappingURL=image-library.js.map