export const VENDOR_ELGATO = 0x0fd9;
export const PRODUCT_ELGATO_STREAMDECK = 0x0060;
export const PRODUCT_ELGATO_STREAMDECK_MINI = 0x0063;
const DEVICE_MODELS_KNOWN = {
    [VENDOR_ELGATO]: {
        [PRODUCT_ELGATO_STREAMDECK]: {
            import: "./models/elgato-stream-deck",
            productName: "Stream Deck",
            vendorName: "Elgato",
        },
        [PRODUCT_ELGATO_STREAMDECK_MINI]: {
            import: "./models/elgato-stream-deck-mini",
            productName: "Stream Deck Mini",
            vendorName: "Elgato",
        },
    },
};
export const DEVICE_MODELS = Object.keys(DEVICE_MODELS_KNOWN)
    .reduce((p, k) => {
    p[k] = Object.assign({}, DEVICE_MODELS_KNOWN[k]);
    return p;
}, {});
/**
 * Register a `StreamDeck` compatable product.
 *
 * @param {number} vendor The numeric vendorId of the product.
 * @param {number} product The numeric productId of the product.
 * @param {Object} config The metadata descriptor for the product.
 * @returns {?Object} undefined, or the previous configuration if one existed.
 */
export function registerStreamDeckProduct(vendor, product, config) {
    if (!DEVICE_MODELS[vendor]) {
        DEVICE_MODELS[vendor] = {};
    }
    const prev = DEVICE_MODELS[vendor][product];
    DEVICE_MODELS[vendor][product] = config;
    return prev;
}
/**
 * Get a previously registered `StreamDeck` compatable product.
 *
 * @param {number} vendor The numeric vendorId of the product.
 * @param {number} product The numeric productId of the product.
 * @returns {?Object} undefined, or the configuration if one existed.
 */
export function getStreamDeckProduct(vendor, product) {
    if (!DEVICE_MODELS[vendor]) {
        return undefined;
    }
    return DEVICE_MODELS[vendor][product];
}
//# sourceMappingURL=models.js.map