import { StreamDeck as StreamDeckBase } from "./stream-deck";
export { IImageLibrary, IImageLibraryCreator, IImageLibraryExtract, IImageRawOptions } from "./image-library";
export { devices, setHidAsyncType, HidAsyncType } from "./hid/hid-async";
export { StreamDeck as StreamDeckBase } from "./stream-deck";
export * from "./models";
export interface StreamDeck extends StreamDeckBase {
    on(event: "down" | "up", cb: (keyIndex: number) => any): this;
    on(event: "error", cb: (err: Error) => any): this;
    once(event: "down" | "up", cb: (keyIndex: number) => any): this;
    once(event: "error", cb: (err: Error) => any): this;
    off(event: "down" | "up", cb: (keyIndex: number) => any): this;
    off(event: "error", cb: (err: Error) => any): this;
}
/**
 * Select the first identified `StreamDeck`.
 * Optional vendor and product id could be used to narrow the search to a specific device.
 *
 * @param {number?} vendor optional vendor identity number
 * @param {number?} product optional product identity number
 * @returns {Promise<StreamDeck>?} the first supported `StreamDeck` within the vendor and product search space, or null
 */
export declare function selectDevice(vendor?: number, product?: number): Promise<StreamDeck | null>;
/**
 * Select all the identified `StreamDeck`s.
 * Optional vendor and product id could be used to narrow the search to range of devices.
 *
 * @param {number?} vendor optional vendor identity number
 * @param {number?} product optional product identity number
 * @returns {Array<Promise<StreamDeck>>} a list of promises to `StreamDeck`s within the vendor and product search space
 */
export declare function selectAllDevices(vendor?: number, product?: number): Promise<Array<Promise<StreamDeck>>>;
