"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const hid_async_1 = require("./hid/hid-async");
const models_1 = require("./models");
var hid_async_2 = require("./hid/hid-async");
exports.devices = hid_async_2.devices;
exports.setHidAsyncType = hid_async_2.setHidAsyncType;
var stream_deck_1 = require("./stream-deck");
exports.StreamDeckBase = stream_deck_1.StreamDeck;
__export(require("./models"));
/**
 * Select the first identified `StreamDeck`.
 * Optional vendor and product id could be used to narrow the search to a specific device.
 *
 * @param {number?} vendor optional vendor identity number
 * @param {number?} product optional product identity number
 * @returns {Promise<StreamDeck>?} the first supported `StreamDeck` within the vendor and product search space, or null
 */
async function selectDevice(vendor, product) {
    const devs = await hid_async_1.devices();
    for (const device of devs) {
        if (vendor === undefined || vendor === device.vendorId) {
            const models = models_1.DEVICE_MODELS[device.vendorId];
            if (models) {
                if (product === undefined || product === device.productId) {
                    const productModule = models[device.productId];
                    if (productModule) {
                        const m = await Promise.resolve().then(() => __importStar(require(productModule.import)));
                        return new m.default(device.path);
                    }
                    else if (product !== undefined) {
                        const m = "No implementations for the product " + product + " of vendor " + vendor;
                        const e = new Error(m);
                        e.code = "STRMDCK_MISSING_PRODUCT";
                        throw e;
                    }
                }
            }
            else if (vendor !== undefined) {
                const m = "No implementations for any product of vendor " + vendor;
                const e = new Error(m);
                e.code = "STRMDCK_MISSING_VENDOR";
                throw e;
            }
        }
    }
    return null;
}
exports.selectDevice = selectDevice;
/**
 * Select all the identified `StreamDeck`s.
 * Optional vendor and product id could be used to narrow the search to range of devices.
 *
 * @param {number?} vendor optional vendor identity number
 * @param {number?} product optional product identity number
 * @returns {Array<Promise<StreamDeck>>} a list of promises to `StreamDeck`s within the vendor and product search space
 */
async function selectAllDevices(vendor, product) {
    const res = [];
    const devs = await hid_async_1.devices();
    for (const device of devs) {
        if (vendor === undefined || vendor === device.vendorId) {
            const models = models_1.DEVICE_MODELS[device.vendorId];
            if (models) {
                if (product === undefined || product === device.productId) {
                    const productModule = models[device.productId];
                    if (productModule) {
                        res.push(Promise.resolve().then(() => __importStar(require(productModule.import))).then((m) => new m.default(device.path)));
                    }
                    else if (product !== undefined) {
                        const m = "No implementations for the product " + product + " of vendor " + vendor;
                        const e = new Error(m);
                        e.code = "STRMDCK_MISSING_PRODUCT";
                        throw e;
                    }
                }
            }
            else if (vendor !== undefined) {
                const m = "No implementations for any product of vendor " + vendor;
                const e = new Error(m);
                e.code = "STRMDCK_MISSING_VENDOR";
                throw e;
            }
        }
    }
    return res;
}
exports.selectAllDevices = selectAllDevices;
//# sourceMappingURL=exports.js.map