export interface IStreamDeckMetaData {
    import: string;
    productName: string;
    vendorName: string;
}
export interface IStreamDeckDevices {
    [vendor: number]: {
        [product: number]: IStreamDeckMetaData;
    };
}
export declare const VENDOR_ELGATO = 4057;
export declare const PRODUCT_ELGATO_STREAMDECK = 96;
export declare const PRODUCT_ELGATO_STREAMDECK_MINI = 99;
declare const DEVICE_MODELS_KNOWN: {
    [VENDOR_ELGATO]: {
        [PRODUCT_ELGATO_STREAMDECK]: {
            import: string;
            productName: string;
            vendorName: string;
        };
        [PRODUCT_ELGATO_STREAMDECK_MINI]: {
            import: string;
            productName: string;
            vendorName: string;
        };
    };
};
export declare const DEVICE_MODELS: IStreamDeckDevices & typeof DEVICE_MODELS_KNOWN;
/**
 * Register a `StreamDeck` compatable product.
 *
 * @param {number} vendor The numeric vendorId of the product.
 * @param {number} product The numeric productId of the product.
 * @param {Object} config The metadata descriptor for the product.
 * @returns {?Object} undefined, or the previous configuration if one existed.
 */
export declare function registerStreamDeckProduct(vendor: number, product: number, config: IStreamDeckMetaData): IStreamDeckMetaData | undefined;
/**
 * Get a previously registered `StreamDeck` compatable product.
 *
 * @param {number} vendor The numeric vendorId of the product.
 * @param {number} product The numeric productId of the product.
 * @returns {?Object} undefined, or the configuration if one existed.
 */
export declare function getStreamDeckProduct(vendor: number, product: number): IStreamDeckMetaData | undefined;
export {};
