"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.VENDOR_ELGATO = 0x0fd9;
exports.PRODUCT_ELGATO_STREAMDECK = 0x0060;
exports.PRODUCT_ELGATO_STREAMDECK_MINI = 0x0063;
const DEVICE_MODELS_KNOWN = {
    [exports.VENDOR_ELGATO]: {
        [exports.PRODUCT_ELGATO_STREAMDECK]: {
            import: "./models/elgato-stream-deck",
            productName: "Stream Deck",
            vendorName: "Elgato",
        },
        [exports.PRODUCT_ELGATO_STREAMDECK_MINI]: {
            import: "./models/elgato-stream-deck-mini",
            productName: "Stream Deck Mini",
            vendorName: "Elgato",
        },
    },
};
exports.DEVICE_MODELS = Object.keys(DEVICE_MODELS_KNOWN)
    .reduce((p, k) => {
    p[k] = Object.assign({}, DEVICE_MODELS_KNOWN[k]);
    return p;
}, {});
/**
 * Register a `StreamDeck` compatable product.
 *
 * @param {number} vendor The numeric vendorId of the product.
 * @param {number} product The numeric productId of the product.
 * @param {Object} config The metadata descriptor for the product.
 * @returns {?Object} undefined, or the previous configuration if one existed.
 */
function registerStreamDeckProduct(vendor, product, config) {
    if (!exports.DEVICE_MODELS[vendor]) {
        exports.DEVICE_MODELS[vendor] = {};
    }
    const prev = exports.DEVICE_MODELS[vendor][product];
    exports.DEVICE_MODELS[vendor][product] = config;
    return prev;
}
exports.registerStreamDeckProduct = registerStreamDeckProduct;
/**
 * Get a previously registered `StreamDeck` compatable product.
 *
 * @param {number} vendor The numeric vendorId of the product.
 * @param {number} product The numeric productId of the product.
 * @returns {?Object} undefined, or the configuration if one existed.
 */
function getStreamDeckProduct(vendor, product) {
    if (!exports.DEVICE_MODELS[vendor]) {
        return undefined;
    }
    return exports.DEVICE_MODELS[vendor][product];
}
exports.getStreamDeckProduct = getStreamDeckProduct;
//# sourceMappingURL=models.js.map