"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stream_deck_1 = require("../stream-deck");
const NUM_FIRST_PAGE_PIXELS = 2583;
const NUM_SECOND_PAGE_PIXELS = 2601;
const NUM_FIRST_PAGE_PIXEL_BYTES = NUM_FIRST_PAGE_PIXELS * 3;
const NUM_SECOND_PAGE_PIXEL_BYTES = NUM_SECOND_PAGE_PIXELS * 3;
const NUM_TOTAL_PIXEL_BYTES = NUM_FIRST_PAGE_PIXEL_BYTES + NUM_SECOND_PAGE_PIXEL_BYTES;
exports.DEV_VENDOR = 0x0fd9;
exports.DEV_PRODUCT = 0x0060;
class ElgatoStreamDeck extends stream_deck_1.StreamDeck {
    constructor(devicePath) {
        super(devicePath);
        this.iconSize = 72;
        this.buttonColumns = 5;
        this.buttonRows = 3;
        this.buttonLength = 3 * 5;
        this.buttonLayout = [
            [4, 3, 2, 1, 0],
            [9, 8, 7, 6, 5],
            [14, 13, 12, 11, 10],
        ];
        this.pagePacketSize = 8191;
    }
    writeImagePage(keyIndex, pixels) {
        if (pixels.length !== NUM_TOTAL_PIXEL_BYTES) {
            throw Error("Unexpected amounts of bytes, got " + pixels + " but expected " + NUM_TOTAL_PIXEL_BYTES);
        }
        const a = this._writePage1(keyIndex, pixels);
        const b = this._writePage2(keyIndex, pixels);
        return Promise.all([a, b]).then(([x, y]) => x + y);
    }
    /**
     * Writes a Stream Deck's page 1 headers and image data to the Stream Deck.
     *
     * @private
     * @param {number} keyIndex The key to write to 0 - 14
     * @param {Uint8Array} pixels Full image data for extraction to page 1
     * @returns {Promise<number>}
     */
    _writePage1(keyIndex, pixels) {
        const header = Buffer.from([
            0x02, 0x01, 0x01, 0x00, 0x00, keyIndex + 1, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x42, 0x4d, 0xf6, 0x3c, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x36, 0x00, 0x00, 0x00, 0x28, 0x00,
            0x00, 0x00, 0x48, 0x00, 0x00, 0x00, 0x48, 0x00,
            0x00, 0x00, 0x01, 0x00, 0x18, 0x00, 0x00, 0x00,
            0x00, 0x00, 0xc0, 0x3c, 0x00, 0x00, 0xc4, 0x0e,
            0x00, 0x00, 0xc4, 0x0e, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        ]);
        const packet = Buffer.alloc(this.pagePacketSize);
        packet.set(header);
        packet.set(pixels.subarray(0, NUM_FIRST_PAGE_PIXEL_BYTES), header.length);
        return this.write(packet);
    }
    /**
     * Writes a Stream Deck's page 2 headers and image data to the Stream Deck.
     *
     * @private
     * @param {number} keyIndex The key to write to 0 - 14
     * @param {Uint8Array} pixels Image data for page 2
     * @returns {Promise<number>}
     */
    _writePage2(keyIndex, pixels) {
        const header = Buffer.from([
            0x02, 0x01, 0x02, 0x00, 0x01, keyIndex + 1, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        ]);
        const packet = Buffer.alloc(this.pagePacketSize);
        packet.set(header);
        packet.set(pixels.subarray(NUM_FIRST_PAGE_PIXEL_BYTES), header.length);
        return this.write(packet);
    }
}
exports.default = ElgatoStreamDeck;
//# sourceMappingURL=elgato-stream-deck.js.map